####################################################################
# (C) Copyright 2006-2017 Hewlett Packard Enterprise Development LP.
# @(#) serviceguard-xdc UPCC Package Configuration Script.
# @(#) Product Name              :  Extended Distance Cluster on Linux
# @(#) Product Version           :  A.12.10.00
# @(#) Patch Name                :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of serviceguard-xdc.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

# Source in /etc/cmcluster.conf to get the Serviceguard path environment 
# variables. 

. ${SGCONFFILE:=/etc/cmcluster.conf}
XDC_UTILS=${SGCONF}/scripts/ext/xdc_utils.sh

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

. ${XDC_UTILS}
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

###################################################################

#UDEV commands are different on RedHat5
typeset -x TASK=$0
typeset UDEV_TRIGGER="udevadm trigger"
typeset UDEV_INFO="udevadm info -q name -n"
test -f /etc/redhat-release && \
if /bin/grep "Tikanga" /etc/redhat-release >/dev/null 2>&1
then
    UDEV_INFO="udevinfo -q name -n"
    UDEV_TRIGGER="udevtrigger"
fi
   
function verify_physical_data_replication
{
    sg_log 5 "verify_physical_data_replication $1"

    typeset -x ACTION=$1

    sg_log 0 "This package is configured with remote data replication."
    if [[ -x ${SGSBIN}/raid_control ]]
    then
        if [[ "${PKG_DATA_REPLICATION_METHOD}" == "LVM" ]]; then
           exec ${SGSBIN}/raid_control ${ACTION}
        elif [[ "${PKG_DATA_REPLICATION_METHOD}" == "VXVM" ]]; then
           exec ${SGSBIN}/raid_control ${ACTION}
        else
           exec ${SGSBIN}/raid_control ${ACTION} "${SG_VGCHANGE_CMD}" "${SG_VG[*]}"
        fi 
    else
        sg_log 0 "ERROR: The package is configured to use MD Software RAID"
        sg_log 0 "\tremote data replication, but $SGSBIN/raid_control"
        sg_log 0 "\tis either non-existent or non-executable."
        sg_log 0 "\tPlease install the Extended Distance Cluster for SGLX correctly."
        if sg_check_if_it_is_reconfiguration
        then
            sg_log 999 "ERROR: The package is configured to use MD Software RAID"
            sg_log 999 "\tremote data replication, but $SGSBIN/raid_control"
            sg_log 999 "\tis either non-existent or non-executable."
            sg_set_reconfig_failure
        fi 
        exit 1
    fi

    typeset -i exit_val=$?

    if [[ $exit_val -ne 0 ]]
    then
        if [[ $exit_val -eq 1 ]]
        then
            sg_log 0 "ERROR: The package cannot $ACTION data replication on any node in the cluster."
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: The package cannot $ACTION data replication on any node in the cluster."
                sg_set_reconfig_failure
            fi
        else
            sg_log 0 "ERROR: The package cannot $ACTION data replication on this node."
            if sg_check_if_it_is_reconfiguration
            then 
                sg_log 999 "ERROR: The package cannot $ACTION data replication on this node."
                sg_set_reconfig_failure
            fi
        fi
    to_exit=$exit_val
    return
    fi
}

##########################################################################
#
# is_package_up() is used to determine whether the package is running 
# on the current node.
# If package is unowned or not running on current node it returns 1.
# If package is running on the current node it returns 0.
#
##########################################################################

function is_package_up
{
    sg_log 5 "is_package_up"

    typeset owner=""
    typeset current_node=""
    
    # Check whether the package is running on the current node
    owner=`cmviewcl -f line -p $SG_PACKAGE_NAME 2>/dev/null | grep "^owner="`
    owner=${owner#*=}

    # get the hostname of the current node
    current_node=$(echo $(uname -n) | sed -e 's/\..*$//')
    if (( $? != 0 )) ; then
       sg_log 0 "ERROR: Function is_package_up"
       sg_log 0 "       Error in getting the hostname"
       to_exit=1
    fi

    if [ "$current_node" == "$owner" ]; then
        return 0
    else
	return 1
    fi
}

#######################################################################
# is_valid_configuration determines whether the configuration changes
# are valid. Only the following changes are allowed online
# 1. Adding an MD 
# 2. Removing an MD
# 3. Replacing one mirror half in an MD
# Following changes are not allowed
# 1. Addition and deletion of MDs in a single cmapplyconf
# 2. Addition/deletion of an MD along with replacing a mirror half in
#    another MD
# 3. Replacing both the mirror halves of an MD in the same cmapplyconf
#######################################################################

function is_valid_configuration
{
    typeset -i add_md_flag=0
    typeset -i remove_md_flag=0
    typeset -i add_dev_flag=0
    typeset -i remove_dev_flag=0
    typeset -i found=0
    typeset -i prev_num_raid=${#PREV_XDC_RAID_DEVICE[*]}
    typeset -i a_raid=0
    typeset DEVICES=
    typeset added_md_names=
    typeset removed_md_names=
    typeset device=""

    typeset -i num_raid=${#XDC_RAID_DEVICE[*]}

    while (( $a_raid < $prev_num_raid ))
    do
        found=0
        raid=0
        while (( $raid < $num_raid ))
        do
            if [[ "${PREV_XDC_RAID_DEVICE[$a_raid]}" = "${XDC_RAID_DEVICE[$raid]}" ]]
            then
                found=1
                device=""
                if [[ "${PREV_XDC_DEVICE_0[$a_raid]}" = "${XDC_DEVICE_0[$raid]}" &&
                    "${PREV_XDC_DEVICE_1[$a_raid]}" != "${XDC_DEVICE_1[$raid]}" ]]
                then
                    (( remove_dev_flag = remove_dev_flag + 1 ))
                    device=${PREV_XDC_DEVICE_1[$a_raid]}
                elif [[ "${PREV_XDC_DEVICE_0[$a_raid]}" != "${XDC_DEVICE_0[$raid]}" &&
                      "${PREV_XDC_DEVICE_1[$a_raid]}" = "${XDC_DEVICE_1[$raid]}" ]]
                then
                    (( remove_dev_flag = remove_dev_flag + 1 ))
                    device=${PREV_XDC_DEVICE_0[$a_raid]}
                elif [[ "${PREV_XDC_DEVICE_0[$a_raid]}" != "${XDC_DEVICE_0[$raid]}" &&
                     "${PREV_XDC_DEVICE_1[$a_raid]}" != "${XDC_DEVICE_1[$raid]}" ]]
                then
                     if [[ "${PREV_XDC_DEVICE_0[$a_raid]}" = "${XDC_DEVICE_1[$raid]}" &&
                        "${PREV_XDC_DEVICE_1[$a_raid]}" != "${XDC_DEVICE_0[$raid]}" ]]
                     then
                        (( remove_dev_flag = remove_dev_flag + 1 ))
                        device=${PREV_XDC_DEVICE_1[$a_raid]}
                     elif [[ "${PREV_XDC_DEVICE_0[$a_raid]}" != "${XDC_DEVICE_1[$raid]}" &&
                          "${PREV_XDC_DEVICE_1[$a_raid]}" = "${XDC_DEVICE_0[$raid]}" ]]
                     then
                        (( remove_dev_flag = remove_dev_flag + 1 ))
                        device=${PREV_XDC_DEVICE_0[$a_raid]}
                     elif [[ "${PREV_XDC_DEVICE_0[$a_raid]}" != "${XDC_DEVICE_1[$raid]}" &&
                          "${PREV_XDC_DEVICE_1[$a_raid]}" != "${XDC_DEVICE_0[$raid]}" ]]
                     then
                        sg_log 0 "ERROR: Removing both mirror halves (${PREV_XDC_DEVICE_0[$a_raid]}, ${PREV_XDC_DEVICE_1[$a_raid]})"\
                                 "of the raid device ${PREV_XDC_RAID_DEVICE[$a_raid]} is not supported."
                        to_exit=1
                        return
                     fi
                fi
                if [ ! -z "$device" ]; then
                    if [ -z "$DEVICES" ]; then
                        DEVICES="$device"
                    else
                        DEVICES=( "${DEVICES[@]}" "$device" )
                    fi
                fi
                break
            else
                (( raid = raid + 1 ))
            fi
        done
        if (( $found == 0 ))
        then
            remove_md_flag=1
            if [ -z "$removed_md_names" ]; then
                removed_md_names=${PREV_XDC_RAID_DEVICE[$a_raid]}
            else
                removed_md_names=( "${removed_md_names[@]}" "${PREV_XDC_RAID_DEVICE[$a_raid]}" )
            fi
        fi
        (( a_raid = a_raid + 1 ))
    done

    raid=0

    # Check for any MD devices added

    while (( $raid < $num_raid ))
    do
        found=0
        a_raid=0
        while (( $a_raid < $prev_num_raid ))
        do
            if [[ "${PREV_XDC_RAID_DEVICE[$a_raid]}" = "${XDC_RAID_DEVICE[$raid]}" ]]
            then
                found=1
                break
            else
                (( a_raid = a_raid + 1 ))
            fi
        done
        if ((found == 0))
        then
            add_md_flag=1
            if [ -z "$added_md_names" ]; then
                added_md_names=${XDC_RAID_DEVICE[$raid]}
            else
                added_md_names=( "${added_md_names[@]}" "${XDC_RAID_DEVICE[$raid]}" )
            fi
        fi
        (( raid = raid + 1 ))
    done

    if (( $remove_md_flag == 1 && $add_md_flag == 1 ))
    then
        sg_log 0 "ERROR: Online addition of MD device(s)($(IFS=, ; echo "${added_md_names[*]}")) and removal of" \
                 "MD device(s)($(IFS=, ; echo "${removed_md_names[*]}")) together is not supported."
        sg_log 0 "Please perform these two steps one after the other."
        to_exit=1
    elif (( $remove_dev_flag > 1 ))
    then
        sg_log 0 "ERROR: Replacing multiple mirror halves($(IFS=, ; echo "${DEVICES[*]}")) together is not supported" \
                 "while package is running."
        sg_log 0 "Please perform these operations one after the other."
        to_exit=1
    elif (( $remove_md_flag == 1 && $remove_dev_flag == 1 ))
    then
        sg_log 0 "ERROR: Online removal of MD device(s)($(IFS=, ; echo "${removed_md_names[*]}")) and replacing" \
                 "mirror halves ($(IFS=, ; echo "${DEVICES[*]}")) together is not supported."
        sg_log 0 "Please perform these two steps one after the other."
        to_exit=1
    elif (( $add_md_flag == 1 && $remove_dev_flag == 1 ))
    then
        sg_log 0 "ERROR: Online addition of MD device(s)($(IFS=, ; echo "${added_md_names[*]}")) and replacing" \
                 "mirror halves ($(IFS=, ; echo "${DEVICES[*]}")) together is not supported."
        sg_log 0 "Please perform these two steps one after the other."
        to_exit=1
    fi
}
##########################################################################
#
# xdc_validate_command() is called while validating the package
# configuration (through cmcheckconf/cmapplyconf)
#
# The validation of xdc consists of the following steps:
# 1. Activate all md raid devices.
# 2. Validate vgs and lvs defined in the package.
# 3. Deactivate all md raid devices
#
# cmapplyconf may be used either when applying a new package or
# reapplying the package configuration after change. In the first 
# case, since the package is not running, md devices will be activated
# on all nodes in the cluster with a single component disk. After the 
# vg and lv validation, the md raid devices will be deactivated by xdc_val.sh.
# In the second case however, since md will be already active on the 
# node running the package, the validation code will activatate the md raid 
# devices on all nodes, except the one running the package, with a single disk. 
# After this the validation will happen as usual and then md raid devices will be 
# deactivated on all nodes except the one running the package by xdc_val.sh.
#
##########################################################################

function xdc_validate_command
{
    
    typeset -i raid_up=0
    typeset -i package_up=0
    typeset -i vg_found=0
    typeset CM_CHKDISK="${SGSBIN}/cmcheckdisk"
    typeset MD_RAIDSTART="mdadm -A -R"
    typeset MD_STATE_DIR="$SGROOT/xdc/mdstate"   
    typeset -i num_vmdk=0

    sg_log 5 "xdc_validate_command"
    
    sg_check_module_in_pkg $VMFS_MODULE
    if [ $? -eq 0 ]; then
        num_vmdk=${#SG_VMDK_FILE_NAME[*]}    
    fi
    
    if [[ $num_vmdk -eq 1 ]]
    then
        sg_log 0 "ERROR: There is only one VMDK filename specified. For XDC, atleast two"\
        "VMDK files must be specified."
        to_exit=1
        return
    fi
    
    if sg_check_if_storage_module_skip; then    
       sg_log 5 "INFO: Validation of XDC module will be skipped"\
       "on this node"
       to_exit=0
       return
    fi       

    if [[ $num_vmdk -ge 1 ]]
    then
        $UDEV_TRIGGER
    fi
    
    # Check  whether the package is running on the current node
    is_package_up
    if (( $? == 0 )); then
	package_up=1
    fi
	
    for ((  I = 0 ;  I < ${#XDC_RAID_DEVICE[@]};  I++  ))
    do
	# Set the path to the status file for XDC_RAID_DEVICE
	typeset MD_STATUS_FILE="$MD_STATE_DIR/$(basename ${XDC_RAID_DEVICE[$I]})"

	# Check whether the md raid device is already active on the current node
	grep -w "$(basename ${XDC_RAID_DEVICE[$I]})" /proc/mdstat >/dev/null 2>&1
	if (( $? == 0 )); then
	   raid_up=1
	fi
	
	# If the md raid device is running on current node but the package is not
	# up on this node, log an error message to make sure that the md raid device 
	# is stopped before re-applying the package. 
	if (( $raid_up == 1 && $package_up == 0)); then
	   sg_log 0 "ERROR: Raid Device ${XDC_RAID_DEVICE[$I]} is active on node $current_node"
	   sg_log 0 "       but the package $SG_PACKAGE_NAME is not running. Stop the raid device"
	   sg_log 0 "       before running cmapplyconf/cmcheckconf"
	   to_exit=1
	   break
	fi 
	 
        # Start the md raid devices with a single clean disk
    	# This is done to prevent any chance of a disk resync  during validation on 
	# multiple nodes.

	# Try to start with the first component device	
	if (( $raid_up == 0 )); then
	   # Check XDC_DEVICE_0. Start only if it is clean
	   grep -w "${XDC_DEVICE_0[$I]}" $MD_STATUS_FILE >/dev/null 2>&1
	   if (( $? != 0 )); then
	      $CM_CHKDISK /dev/$($UDEV_INFO ${XDC_DEVICE_0[$I]})
	      if (( $? == 0 )); then
		  $MD_RAIDSTART ${XDC_RAID_DEVICE[$I]} ${XDC_DEVICE_0[$I]} >/dev/null 2>&1
		  if (( $? == 0 )); then
		      raid_up=1
		  fi
	      fi
	   fi
	fi

	# If the activation with first component device is not possible
	# try to start with the second component device.
	if (( $raid_up == 0 )); then
	   # Check XDC_DEVICE_1. Start only if it is clean
	   grep -w "${XDC_DEVICE_1[$I]}" $MD_STATUS_FILE >/dev/null 2>&1
	   if (( $? != 0 )); then
	      $CM_CHKDISK /dev/$($UDEV_INFO ${XDC_DEVICE_1[$I]})
	      if (( $? == 0 )); then
		  $MD_RAIDSTART ${XDC_RAID_DEVICE[$I]} ${XDC_DEVICE_1[$I]} >/dev/null 2>&1
		  if (( $? == 0 )); then
		     raid_up=1
		  fi
	      fi
	   fi
	fi

	if (( $raid_up != 1 )); then
	   sg_log 5 "xdc_validate_command"
	   sg_log 5 "ERROR: Failed to activate ${XDC_RAID_DEVICE[$I]} on node $current_node"
	   to_exit=1
	   break
	else
           raid_up=0
           vg=$(pvdisplay -C --noheadings -o vg_name ${XDC_RAID_DEVICE[$I]} 2>/dev/null | sed 's/ //g')
           hash_found=$(echo ${vg} | grep "\#" 2>/dev/null)
           if [[ ("$vg" = "")  || ("${hash_found}" != "") ]]; then
              sg_log 0 "ERROR: No volume group is configured on ${XDC_RAID_DEVICE[$I]}. Create volume group"
              sg_log 0 "       on ${XDC_RAID_DEVICE[$I]} and add the volume group in package configuration file"
              sg_log 0 "       before running cmcheckconf/cmapplyconf."
              to_exit=1
              break
           else
              for ((  J = 0 ;  J < ${#SG_VG[@]};  J++  )); do
                  if [[ "${SG_VG[$J]}" = "${vg}" ]]; then
                     vg_found=1
                     break
                  fi
              done
              if (( ${vg_found} == 0 )); then
                 sg_log 0 "ERROR: ${vg} is configured on ${XDC_RAID_DEVICE[$I]}. But ${vg} is not specified"
                 sg_log 0 "       in package configuration file. Add ${vg} in package configuration file"
                 sg_log 0 "       before running cmcheckconf/cmapplyconf."
                 to_exit=1
                 break
              fi
           fi
        fi
        vg_found=0
        unset vg
        unset hash_found
    done

    if (( $package_up == 1 )); then
        is_valid_configuration  
    fi

    return
}

function find_data_replication_mechanism
{
    find_if_pkg_has_mirrored_vg
    if (( $? == 1 )); then  
        if (( ${#XDC_RAID_DEVICE[@]} > 0 )); then
            sg_log 0 "ERROR: Raid devices cannot be specified when Volume group is mirrored"
            exit 1
        else
            DATA_REP="LVM"
            sg_log 0 "DATA_REP:LVM"
        fi
    else
        find_if_pkg_has_mirrored_dg
        if (( $? == 1 )); then
            if (( ${#XDC_RAID_DEVICE[@]} > 0 )); then
                sg_log 0 "ERROR: Raid devices cannot be specified when VxVM volume is mirrored"
                exit 1
            else
                DATA_REP="VXVM" 
                sg_log 0 "DATA_REP:VXVM"
            fi
        else
            if (( ${#XDC_RAID_DEVICE[@]} > 0 )); then
                DATA_REP="MD"
                sg_log 0 "DATA_REP:MD"
            fi
        fi
    fi
    return
}

function xdc_validate_lvm_mirror
{
  for VG in $MIRRORED_VG_LIST
  do
     LV_LIST=""
     pat="--- Logical volume ---"
     myvar=`lvdisplay $VG`
     for dev in `echo "$myvar" | grep "LV Path" | awk '{print $NF}'`
     do
           lv_found=0
           lvm_mirror_check "$myvar" "$dev" 
           if (( $? == 0 )); then
              for (( I = 0 ; I < ${#SG_FS_NAME[@]}; I++ ))
              do
                  if [[ "${SG_FS_NAME[$I]}" = "${dev}" ]]; then
                     lv_found=1
                     break
                  fi
              done
              if (( ${lv_found} == 1 )); then
                 LV_LIST=$LV_LIST" "$dev
              fi
           fi
     done
     # Check for raid1 mirror type of given LVs.
     var=`lvs --aligned --noheadings --separator :  -a -o name,devices,segtype ${VG}`
     for dev in `echo "$myvar" | grep "LV Path" | awk '{print $NF}'`
     do
         for (( I = 0 ; I < ${#SG_FS_NAME[@]}; I++ ))
         do
            if [[ "${SG_FS_NAME[$I]}" = "${dev}" ]]; then
               lvname=`lvdisplay ${dev} | grep "LV Name" | awk '{print $NF}'`
               mirror_type=`echo "$var" 2>&1 | grep -w ${lvname} | cut -d : -f 3 | tr -d ' ' `
               if [[ "${mirror_type}" != "raid1" ]]; then
                  sg_log 0 "ERROR: Logical volume ${dev} of Volume group ${VG} : Serviceguard-xdc is supported only with raid1 segment type of LVM mirroring"
                   to_exit=1
               fi
            fi
         done
     done

     for LV in $LV_LIST
     do
         no_of_mirrors=`echo "$myvar" 2>&1| awk "/\<$(basename ${LV})\>/ {flag=1;print;next} /--- Logical volume ---/{flag=0} flag {print}" | grep "Mirrored volumes" | awk '{print $NF}'`
         if [[ $no_of_mirrors -gt 2 ]]; then
          sg_log 0 "ERROR: Logical volume ${LV} of Volume group ${VG} : LVM mirroring with more than two mirror legs is not supported with Serviceguard-xdc."
             to_exit=1
         fi
     done
     myvar=""
     var=""
  done

  return
}
 
################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#
typeset -i to_exit=0
typeset DATA_REP=""
typeset MIRRORED_VG_LIST
typeset MIRRORED_VXVM_DG_LIST

case ${1} in
       start)
		verify_physical_data_replication $1 
		;;
        stop)
		verify_physical_data_replication $1 
		;;
    validate)   
                find_data_replication_mechanism
                if [[ "${DATA_REP}" = "MD" ]]; then
                   xdc_validate_command
                elif [[ "${DATA_REP}" = "LVM" ]]; then
                   build_lvm_list
                   xdc_validate_lvm_mirror
                fi
		;;
    reconfigure_stop)
                verify_physical_data_replication $1
                ;;
    reconfigure_start)
                verify_physical_data_replication $1
                ;;
	   *)
          	sg_log 0 "WARNING: Illegal entry point specification $1."
          	;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
