####################################################################
# (C) Copyright 2006-2015 Hewlett Packard Enterprise Development LP.
# @(#) serviceguard-xdc UPCC Package Configuration Script.
# @(#) Product Name              :  Extended Distance Cluster on Linux
# @(#) Product Version           :  A.12.10.00
# @(#) Patch Name                :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of serviceguard-xdc.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

# Source in /etc/cmcluster.conf to get the Serviceguard path environment
# variables.

. ${SGCONFFILE:=/etc/cmcluster.conf}
XDC_UTILS=${SGCONF}/scripts/ext/xdc_utils.sh

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

. ${XDC_UTILS}
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

###################################################################

function xdc_endvalidate_command
{
    typeset MD_RAIDSTOP="mdadm -S"
    typeset owner=""
    typeset current_node=""

    sg_log 5 "xdc_endvalidate_command"
    
    # get the owner of the current package
    owner=`cmviewcl -f line -p $SG_PACKAGE_NAME 2>/dev/null | grep "^owner="`
    owner=${owner#*=}

    # get the hostname of the node
    current_node=$(echo $(uname -n) | sed -e 's/\..*$//')
    if (( $? != 0 )) ; then
       sg_log 0 "ERROR: xdc_endvalidate_command"
       sg_log 0 "       Error in getting the hostname"
       to_exit=1
    fi

    # If the package is running on the current node do not
    # deactivate raid devices , otherwise deactivate the 
    # raid devices after validation is complete.
    if [ "$current_node" != "$owner" ]; then
    	for I in ${XDC_RAID_DEVICE[@]}
        do
            $MD_RAIDSTOP $I >/dev/null 2>&1 
            if (( $? != 0 )) ; then
                sg_log 0 "ERROR: xdc_endvalidate_command"
                sg_log 0 "       Failed to deactivate $I"
                to_exit=1
            fi
        done
    fi
    
    return
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#
# NOTE DO NOT DELETE THESE COMMENTS
# Adding the string "sg_check_if_it_is_reconfiguration" and
# "sg_check_if_reconfig_has_failed". This is to support the
# new online reconfiguration behavior. These strings are added
# as comments. This ensures the check from master_control_script
# that this module can handle the new behavior during online
# configuration.

typeset -i to_exit=0
MD_STATE_DIR="$SGROOT/xdc/mdstate"
DATA_REP_FILE=$MD_STATE_DIR/data_rep_val

case ${1} in
       start)
                # no op
                ;;
        stop)
                # no op
                ;;
    validate) 
                sg_check_if_storage_module_skip
                if [[ $? != 0 ]]; then
                    find_if_pkg_has_mirrored_vg
                    if [[ $? = 0 ]]; then
                       find_if_pkg_has_mirrored_dg
                       if [[ $? = 0 ]]; then
                           xdc_endvalidate_command
                       fi
                    fi
                fi
                ;;
    reconfigure_stop)
                #no op
                ;;
    reconfigure_start)
                #no op
                ;;
           *)
                sg_log 0 "WARNING: Illegal entry point specification $1."
                ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit

